/*
 *  plane.cpp
 *
 *  Author : Ahmed Obied (ahmed.obied@gmail.com)
 *
 */

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Author : Ahmed Obied                                                      //
// Date   : April 4th 2004                                                   //
// Purpose: Create a plane                                                   //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

#include "shape.h"
#include "plane.h"

//-----------------------------------------------------------------------------
// Parameters    : None
// Return value  : None
// Purpose       : The constructor is not used for any special purpose here
// Method called : Sphere
//-----------------------------------------------------------------------------
Plane::Plane()
{

}

//-----------------------------------------------------------------------------
// Parameters    : double d
// Return value  : None
// Purpose       : This function is used to set the distance of the plane from
//                 the coordinate system origin
// Method called : setDistance
//-----------------------------------------------------------------------------
void Plane::setDistance(double d)
{
  distance = d;
}

//-----------------------------------------------------------------------------
// Parameters    : gmVector3 norm
// Return value  : None
// Purpose       : This function is used to set the plane normal
// Method called : setNormal
//-----------------------------------------------------------------------------
void Plane::setNormal(gmVector3 norm)
{
  normal = norm;
}

//-----------------------------------------------------------------------------
// Parameters    : gmVector3 mi
// Return value  : None
// Purpose       : This function is used to to set the minimun value for
//                 the plane's boundry
// Method called : setMin
//-----------------------------------------------------------------------------
void Plane::setMin(gmVector3 mi)
{
  min = mi;
}

//-----------------------------------------------------------------------------
// Parameters    : gmVector3 ma
// Return value  : None
// Purpose       : This function is used to set the maximum value for 
//                 the plane's boundry
// Method called : setMax
//-----------------------------------------------------------------------------
void Plane::setMax(gmVector3 ma)
{
 max = ma;
}

//-----------------------------------------------------------------------------
// Parameters    : gmVector3 p,gmVector3 dir,int &status
// Return value  : None
// Purpose       : This function is used to do a plane ray intersection test
// Method called : intersection
//-----------------------------------------------------------------------------
gmVector3 Plane::intersection(gmVector3 p,gmVector3 dir,int &status)
{
  gmVector3 intersectionPoint;
  double tp;
  double t;
  double temp;
  // caluclate the intersection point if there is a hit with the plane
  tp = dot(normal,dir);
  if (tp == 0)
    status = 0;
  else
    {
      temp = dot(normal,p);
      temp = temp + distance;
      temp = -1 * temp;
      t = (temp/tp);
      if (t<0)
	status = 0;
      else
	{
	  status = 1;
	  intersectionPoint = p + (t*dir);
	  // check boundry's for the plane if we don't need it to be 
	  // infinite
	  if ((intersectionPoint[0]>min[0])
	      &&
	      (intersectionPoint[1]>min[1])
	      &&
	      (intersectionPoint[2]>min[2])
	      &&
	      (intersectionPoint[0]<max[0])
	      &&
	      (intersectionPoint[1]<max[1])
	      &&
	      (intersectionPoint[2]<max[2])
	      )
	    status = 1;
	  else
	  status = 0;
	  
	  if (status)
	    {
	  if (tp<0)
	    normal = normal;
	  else
	    normal = -1 * normal;
	    }
	}
    }
  return intersectionPoint;
}

//-----------------------------------------------------------------------------
// Parameters    : gmVector3 point
// Return value  : None
// Purpose       : This function is used to return the plane normal
// Method called : getNormalVector
//-----------------------------------------------------------------------------
gmVector3 Plane::getNormalVector(gmVector3 point)
{
  return normal;
}

//-----------------------------------------------------------------------------
// Parameters    : gmVector3 dir,gmVector3 normal
// Return value  : gmVector3
// Purpose       : This function is used to calculate the reflection vector
//                 for the intersection point
// Method called : getReflectionVector
//-----------------------------------------------------------------------------
gmVector3 Plane::getReflectionVector(gmVector3 dir,gmVector3 normal)
{
  gmVector3 rVec;
  gmVector3 temp;
  double temp_1;
  // find the cosine angle
  temp_1 = dot(dir,normal);
  // extend the angle
  temp = -1 * 2 * normal * temp_1;
  // compute the reflection vector and normalize
  rVec = temp + dir;
  rVec.normalize();
  // return the reflection vector
  return rVec;
}

//-----------------------------------------------------------------------------
// Parameters    : Normal
// Return value  : gmVector3
// Purpose       : This function is used to calculate the transmission vector
// Method called : getTransmissionVector
//----------------------------------------------------------------------------
gmVector3 Plane::getTransmissionVector()
{
  gmVector3 tVec;
  // due to the lack of time, I have not implemented refraction
  return tVec;
}

//-----------------------------------------------------------------------------
// Parameters    : Color amb,Color dif,Color spec,
//		   double ref, double trans, int shininess
// Return value  : None
// Purpose       : This function is used to set the material of the plane
// Method called : setMat
//-----------------------------------------------------------------------------
void Plane::setMat(Color amb,Color dif,Color spec,
		   double ref, double trans, int shininess)
{
  shapeMat.setAmbient(amb);
  shapeMat.setDiffuse(dif);
  shapeMat.setSpecular(spec);
  shapeMat.setReflection(ref);
  shapeMat.setTransmission(trans);
  shapeMat.setShininess(shininess);
}

//-----------------------------------------------------------------------------
// Parameters    : None
// Return value  : None
// Purpose       : This function is used to get the material for the plane
// Method called : getMat
//-----------------------------------------------------------------------------
Material Plane::getMat()
{
  return shapeMat;
}
